/*
 * tests.cpp
 *
 *  Created on: Oct 14, 2008
 *      Author: bloch
 */

#include <iostream>
using std::cout;
using std::endl;

#include "EmptyIntList.h"
#include "NonEmptyIntList.h"
#include "EmptyFloatList.h"
#include "NonEmptyFloatList.h"

NonEmptyFloatList *forceFloat(FloatList *blah)
{
	return dynamic_cast<NonEmptyFloatList *>(blah);
}

NonEmptyIntList *forceInt(IntList *blah)
{
	return dynamic_cast<NonEmptyIntList *>(blah);
}

void testFloats();
void testInts();

int main () {
	testFloats();
	testInts();
	return 0;
}

void testFloats()
{
	FloatList *ex0 = new EmptyFloatList();
	FloatList *ex1 = new NonEmptyFloatList(3.2, ex0);
	FloatList *ex2 = new NonEmptyFloatList(-7.1, ex1);

	cout << "\nTesting floats:" << endl;
	cout << "ex0->isEmpty(): " << ex0->isEmpty() << " should be true" << endl;
	cout << "ex1->isEmpty(): " << ex1->isEmpty() << " should be false" << endl;
	cout << "ex2->isEmpty(): " << ex2->isEmpty() << " should be false" << endl;

	cout << "ex1->getFirst(): " << dynamic_cast<NonEmptyFloatList *>(ex1)->getFirst() << " should be 3.2 " << endl;

	// That's a pain.  Let's put that cast into a function:
	NonEmptyFloatList *ex1again = forceFloat(ex1);
	NonEmptyFloatList *ex2again = forceFloat(ex2);

	cout << "ex1again->getFirst(): " << ex1again->getFirst() << " should be 3.2" << endl;
	cout << "ex2again->getFirst(): " << ex2again->getFirst() << " should be -7.1" << endl;
	cout << "ex2again->getRest()->getFirst(): " <<
		forceFloat(ex2again->getRest())->getFirst() << " should be 3.2" << endl;
	cout << "ex0->length(): " << ex0->length() << " should be 0" << endl;
	cout << "ex1->length(): " << ex1->length() << " should be 1" << endl;
	cout << "ex2->length(): " << ex2->length() << " should be 2" << endl;

	cout << "ex0: " << (*ex0) << " should be []" << endl;
	cout << "ex1: " << (*ex1) << " should be [3.2]" << endl;
	cout << "ex2: " << (*ex2) << " should be [-7.1, 3.2]" << endl;

	FloatList *bigex =
		new NonEmptyFloatList (12.73,
				new NonEmptyFloatList (-2.851,
						new NonEmptyFloatList (-73.03,
								new NonEmptyFloatList (0,
										new NonEmptyFloatList (3.843,
												new EmptyFloatList())))));
	cout << "bigex->getRest()->getRest()->getFirst(): " <<
		forceFloat(forceFloat(forceFloat(bigex)->getRest())->getRest())->getFirst() <<
		" should be -73.03" << endl;
	cout << "bigex: " << *bigex << " should be [12.73, -2.851, -73.03, 0, 3.843]" << endl;
	cout << "bigex->length(): " << bigex->length() << " should be 5" << endl;

	/*
	cout << "ex1again->getRest()->getFirst(): " <<
		forceFloat(ex1again->getRest())->getFirst() <<
		" should throw an exception because ex1again->getRest() is empty" << endl;
	cout << "We shouldn't get here at all!" << endl;
	*/
}

void testInts()
{
	IntList *ex0 = new EmptyIntList();
	IntList *ex1 = new NonEmptyIntList(3, ex0);
	IntList *ex2 = new NonEmptyIntList(-7, ex1);

	cout << "\nTesting ints:" << endl;
	cout << "ex0->isEmpty(): " << ex0->isEmpty() << " should be true" << endl;
	cout << "ex1->isEmpty(): " << ex1->isEmpty() << " should be false" << endl;
	cout << "ex2->isEmpty(): " << ex2->isEmpty() << " should be false" << endl;

	cout << "ex1->getFirst(): " << dynamic_cast<NonEmptyIntList *>(ex1)->getFirst() << " should be 3 " << endl;

	// That's a pain.  Let's put that cast into a function:
	NonEmptyIntList *ex1again = forceInt(ex1);
	NonEmptyIntList *ex2again = forceInt(ex2);

	cout << "ex1again->getFirst(): " << ex1again->getFirst() << " should be 3" << endl;
	cout << "ex2again->getFirst(): " << ex2again->getFirst() << " should be -7" << endl;
	cout << "ex2again->getRest()->getFirst(): " <<
		forceInt(ex2again->getRest())->getFirst() << " should be 3" << endl;
	cout << "ex0->length(): " << ex0->length() << " should be 0" << endl;
	cout << "ex1->length(): " << ex1->length() << " should be 1" << endl;
	cout << "ex2->length(): " << ex2->length() << " should be 2" << endl;

	cout << "ex0: " << (*ex0) << " should be []" << endl;
	cout << "ex1: " << (*ex1) << " should be [3]" << endl;
	cout << "ex2: " << (*ex2) << " should be [-7, 3]" << endl;

	IntList *bigex =
		new NonEmptyIntList (12,
				new NonEmptyIntList (-2,
						new NonEmptyIntList (-73,
								new NonEmptyIntList (0,
										new NonEmptyIntList (3,
												new EmptyIntList())))));
	cout << "bigex->getRest()->getRest()->getFirst(): " <<
		forceInt(forceInt(forceInt(bigex)->getRest())->getRest())->getFirst() <<
		" should be -73" << endl;
	cout << "bigex: " << *bigex << " should be [12, -2, -73, 0, 3]" << endl;
	cout << "bigex->length(): " << bigex->length() << " should be 5" << endl;

/*
	cout << "ex1again->getRest()->getFirst(): " <<
		forceInt(ex1again->getRest())->getFirst() <<
		" should throw an exception because ex1again->getRest() is empty" << endl;
	cout << "We shouldn't get here at all!" << endl;
	*/
}
